@@ -86,7 +86,7 @@ Memento::Memento(not_null<PeerData*> peer, PeerId migratedPeerId, Type type)
 }
 
 Memento::Memento(not_null<Data::ForumTopic*> topic, Type type)
-: Memento(topic->channel(), topic, nullptr, PeerId(), type) {
+: Memento(topic->peer(), topic, nullptr, PeerId(), type) {
 }
 
 Memento::Memento(not_null<Data::SavedSublist*> sublist, Type type)
